﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows.Forms;
using HealthyFood.Business;

namespace HealthyFood
{
	public partial class MainForm : Form
	{
		public MainForm()
		{
			InitializeComponent();

			FillProducers();
		}

		private void FillProducers(int? selectedProducerID = null)
		{
			List<Producer> producers = Program.Session.GetProducers().ToList();
			lbProducers.DataSource = producers;
			lbProducers.DisplayMember = "FullText";
			lbProducers.ValueMember = "ProducerID";
			if (selectedProducerID != null)
				lbProducers.SelectedValue = selectedProducerID;
		}

		private void FillBrands(int? selectedBrandID = null)
		{
			lbBrands.DataSource = null;

			if (lbProducers.SelectedItem != null)
			{
				var producer = (Producer)lbProducers.SelectedItem;
				List<Brand> brands = producer.GetBrands().ToList();
				lbBrands.DataSource = brands;
				lbBrands.DisplayMember = "Name";
				lbBrands.ValueMember = "BrandID";
				if (selectedBrandID != null)
					lbBrands.SelectedValue = selectedBrandID;
			}
		}

		private void FillProducts(int? selectedProductID = null)
		{
			lvProducts.Items.Clear();

			if (lbBrands.SelectedItem != null)
			{
				var brand = (Brand)lbBrands.SelectedItem;
				IEnumerable<Product> products = brand.GetProducts();
				foreach (Product product in products)
				{
					var listItem = new ListViewItem(product.Name);
					listItem.SubItems.Add(product.EnergyValue.ToString());
					listItem.SubItems.Add(product.Proteins == null ? "-" : product.Proteins.ToString());
					listItem.SubItems.Add(product.Fats == null ? "-" : product.Fats.ToString());
					listItem.SubItems.Add(product.Carbohydrates == null ? "-" : product.Carbohydrates.ToString());
					listItem.Tag = product;
					lvProducts.Items.Add(listItem);
					if (selectedProductID != null && product.ProductID == selectedProductID)
						listItem.Selected = true;
				}
			}

			if (lvProducts.Items.Count > 0 && lvProducts.SelectedItems.Count == 0)
				lvProducts.Items[0].Selected = true;
			else
				OnSelectedProductChanged();
		}

		private void OnSelectedProducerChanged()
		{
			bool hasSelectedProducer = lbProducers.SelectedItem != null;
			btnEditProducer.Enabled = hasSelectedProducer;
			btnAddBrand.Enabled = hasSelectedProducer;

			FillBrands();
		}

		private void OnSelectedBrandChaged()
		{
			bool hasSelectedBrand = lbBrands.SelectedItem != null;
			btnEditBrand.Enabled = hasSelectedBrand;
			btnAddProduct.Enabled = hasSelectedBrand;

			lblBrandDescription.Text = (hasSelectedBrand
											? ((Brand)lbBrands.SelectedItem).Description
											: null);

			FillProducts();
		}

		private void OnSelectedProductChanged()
		{
			bool hasSelectedProduct = lvProducts.SelectedItems.Count > 0;
			btnEditProduct.Enabled = hasSelectedProduct;

			if (hasSelectedProduct)
			{
				var product = (Product)lvProducts.SelectedItems[0].Tag;
				IEnumerable<ProductIngredient> productIngredients = product.GetIngredients();
				string[] ingredientTexts = productIngredients.Select(pi => pi.FullText).ToArray();
				lblProductIngredients.Text = string.Join(", ", ingredientTexts);
			}
			else
				lblProductIngredients.Text = null;
		}

		private void lbProducers_SelectedIndexChanged(object sender, EventArgs e)
		{
			OnSelectedProducerChanged();
		}

		private void lbBrands_SelectedIndexChanged(object sender, EventArgs e)
		{
			OnSelectedBrandChaged();
		}

		private void lvProducts_SelectedIndexChanged(object sender, EventArgs e)
		{
			OnSelectedProductChanged();
		}

		private void btnAddProducer_Click(object sender, EventArgs e)
		{
			var dialog = new ProducerDialog();

			if (dialog.ShowDialog() == DialogResult.OK)
			{
				Producer producer = Program.Session.CreateProducer(dialog.ProducerName, dialog.Country);
				FillProducers(producer.ProducerID);
			}
		}

		private void btnEditProducer_Click(object sender, EventArgs e)
		{
			if (lbProducers.SelectedItem == null)
				return;

			var producer = (Producer)lbProducers.SelectedItem;
			var dialog = new ProducerDialog(producer);

			if (dialog.ShowDialog() == DialogResult.OK)
			{
				producer.Update(dialog.ProducerName, dialog.Country);
				FillProducers(producer.ProducerID);
			}
		}

		private void btnAddBrand_Click(object sender, EventArgs e)
		{
			if (lbProducers.SelectedItem == null)
				return;

			var producer = (Producer)lbProducers.SelectedItem;
			var dialog = new BrandDialog();

			if (dialog.ShowDialog() == DialogResult.OK)
			{
				Brand brand = producer.CreateBrand(dialog.BrandName, dialog.Description);
				FillBrands(brand.BrandID);
			}
		}

		private void btnEditBrand_Click(object sender, EventArgs e)
		{
			if (lbBrands.SelectedItem == null)
				return;

			var brand = (Brand)lbBrands.SelectedItem;
			var dialog = new BrandDialog(brand);

			if (dialog.ShowDialog() == DialogResult.OK)
			{
				brand.Update(dialog.BrandName, dialog.Description);
				FillBrands(brand.BrandID);
			}
		}

		private void btnAddProduct_Click(object sender, EventArgs e)
		{
			if (lbBrands.SelectedItem == null)
				return;

			var brand = (Brand)lbBrands.SelectedItem;
			var dialog = new ProductDialog();

			if (dialog.ShowDialog() == DialogResult.OK)
			{
				Product product = brand.CreateProduct(dialog.ProductName, dialog.EnergyValue,
													  dialog.Proteins, dialog.Fats,
													  dialog.Carbohydrates);
				FillProducts(product.ProductID);
			}
		}

		private void btnEditProduct_Click(object sender, EventArgs e)
		{
			if (lvProducts.SelectedItems.Count == 0)
				return;

			var product = (Product)lvProducts.SelectedItems[0].Tag;
			var dialog = new ProductDialog(product);

			if (dialog.ShowDialog() == DialogResult.OK)
			{
				product.Update(dialog.ProductName, dialog.EnergyValue, dialog.Proteins,
							   dialog.Fats, dialog.Carbohydrates);
				FillProducts(product.ProductID);
			}
		}
	}
}
